/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.installation;

import java.net.URL;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.microkernel.api.communication.JMXService;
import org.ow2.petals.microkernel.api.jbi.management.ComponentInstallationService;
import org.ow2.petals.microkernel.api.jbi.management.InstallationServiceMBean;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * @author Christophe HAMERLING - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = ComponentInstallationService.class))
public class ComponentInstallationServiceImpl implements ComponentInstallationService {

    private LoggingUtil log;

    @Requires(name = "installation")
    private InstallationServiceMBean installationService;

    @Requires(name = "jmx")
    private JMXService jmxService;

    @LifeCycle(on = LifeCycleType.START)
    public void start() {
        this.log = new LoggingUtil(
                java.util.logging.Logger
                        .getLogger(ComponentInstallationService.COMPONENT_LOGGER_NAME));
    }

    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
    }

    public boolean install(URL componentZipURL) {
        this.log.debug("Installing component from " + componentZipURL);
        boolean result = true;
        try {
            MBeanServer jmxServer = jmxService.getLocalJMXServer();

            ObjectName newInstallerName = installationService.loadNewInstaller(componentZipURL
                    .toString());
            if (!newInstallerName.getDomain().equals("error")) {
                Object[] nullObjects = new Object[0];
                String[] nullStrings = new String[0];
                Object o = jmxServer.invoke(newInstallerName, "install", nullObjects, nullStrings);
                if (o instanceof ObjectName) {
                    ObjectName componentName = (ObjectName) o;
                    jmxServer.invoke(componentName, "start", nullObjects, nullStrings);
                } else {
                    throw new Exception(
                            "The install() invokation on the component installer has returned an unexpected object: "
                                    + result);
                }
            } else {
                throw new Exception("Component can't be installed");
            }
        } catch (Throwable e) {
            log.error("Error occured during auto installation: ", e);
            result = false;
        }
        return result;
    }

    public boolean uninstall(String componentId) {
        this.log.debug("Uninstall component " + componentId);
        return installationService.forceUnloadInstaller(componentId);
    }
}
