/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task;

import java.io.File;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.jbi.management.PackageHelper;
import org.ow2.petals.microkernel.api.util.LoggingUtil;

/**
 * This task checks the validity of a package to install.
 * <p>
 * Inputs : unzipped package root {@link File} <br/>
 * Outputs : Entity name (String), {@link Jbi} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Frederic Gardes - EBM WebSourcing
 */
public abstract class AbstractCheckPackageTask extends AbstractLoggableTask {

    public AbstractCheckPackageTask(final LoggingUtil log,
            final ContainerConfiguration containerConfiguration) {
        super(log, containerConfiguration);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.Map)
     */
    public void execute(Context context) throws Exception {
        File packageFile = context.getUnzipRoot();
        Jbi descriptor = PackageHelper.loadDescriptor(packageFile);
        this.checkPackage(descriptor, packageFile);

        context.setDescriptor(descriptor);
        context.setEntityName(this.extractEntityName(descriptor));
        context.setEntityVersion(this.extractEntityVersion(descriptor));

    }

    protected abstract String extractEntityName(Jbi descriptor);
    
    protected String extractEntityVersion(Jbi descriptor) {
        return null;
    }

    protected abstract void checkPackage(Jbi descriptor, File packageFile) throws PetalsException;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.Map)
     */
    public void undo(Context context) throws Exception {
        // Nothing to do
    }
}
