/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.installation.install.sl;

import java.io.File;
import java.io.IOException;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.jbi.management.Context;
import org.ow2.petals.microkernel.api.system.repository.RepositoryService;
import org.ow2.petals.microkernel.api.system.repository.exception.RepositoryException;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.jbi.management.task.AbstractCopyPackageToRepositoryTask;

/**
 * This task copy shared library package into the repository.
 * <p>
 * This task MUST be executed with a {@link Context} containing:
 * <ul>
 * <li>shared library name as entity name,</li>
 * <li>shared library version as entity version,</li>
 * <li>archive as exploded directory,</li>
 * </ul>
 * and returns following data in the {@link Context}:
 * <ul>
 * <li>the URL where the shared library is installed in the repository.</li>
 * </ul>
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 */
public class CreateSLRepositoryTask extends AbstractCopyPackageToRepositoryTask {

    public CreateSLRepositoryTask(final LoggingUtil log, final RepositoryService repositorySrv,
            final ContainerConfiguration containerConfiguration) {
        super(log, repositorySrv, containerConfiguration);
    }

    @Override
    protected File copyEntityPackage(String entityName, String entityVersion, File packageFile)
            throws IOException, RepositoryException {
        return repositoryService.addSharedLibrary(entityName, entityVersion, packageFile);

    }

    @Override
    protected void removeEntityPackage(String entityName, String entityVersion) throws IOException {
        repositoryService.removeSharedLibrary(entityName, entityVersion);
    }

}
