/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.endpoint;

import static org.ow2.petals.microkernel.jbi.Constants.ENDPOINT_NAME;
import static org.ow2.petals.microkernel.jbi.Constants.EPR_NAME;
import static org.ow2.petals.microkernel.jbi.Constants.JBI_NAMESPACE;
import static org.ow2.petals.microkernel.jbi.Constants.SERVICE_NAME;

import javax.xml.namespace.QName;

import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.service.ServiceEndpointImpl;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

/**
 * The JBI implementation of the petals service endpoint.
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class JBIServiceEndpointImpl extends ServiceEndpointImpl implements ServiceEndpoint {

    /**
     * Unique serial UID
     */
    private static final long serialVersionUID = 4487649085296224233L;

    /**
     * 
     */
    public JBIServiceEndpointImpl() {
        super();
        this.setType(EndpointType.INTERNAL);
    }

    /**
     * {@inheritDoc}
     */
    public DocumentFragment getAsReference(QName operationName) {
        Document doc = DocumentBuilders.newDocument();
        if (doc == null) {
            return null;
        }
        DocumentFragment result = doc.createDocumentFragment();
        Element e = doc.createElementNS(JBI_NAMESPACE, EPR_NAME);
        e.setAttribute(SERVICE_NAME, this.getServiceName().getLocalPart());
        e.setAttribute(ENDPOINT_NAME, this.getEndpointName());
        result.appendChild(e);
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public QName[] getInterfaces() {
        return this.getInterfacesName().toArray(new QName[this.getInterfacesName().size()]);
    }
}
