/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.messaging.routing.module;

import java.util.Map;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.jbi.ComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.RouterService;
import org.ow2.petals.microkernel.api.jbi.messaging.RoutingException;
import org.ow2.petals.microkernel.api.jbi.messaging.ServiceEndpoint;
import org.ow2.petals.microkernel.api.util.LoggingUtil;
import org.ow2.petals.microkernel.transport.Transporter;
import org.ow2.petals.microkernel.transport.util.TransportSendContext;

/**
 * @author Roland Naudin - EBM WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SenderModule.class))
public class TransportResolverModule implements SenderModule {

    /**
     * Logger wrapper.
     */
    protected LoggingUtil log;

    /**
     * The Topology service fractal component
     */
    @Requires(name = "configuration")
    protected ConfigurationService configurationService;

    private String localContainerName;

    private short attempt;

    private int delay;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.module.SenderModule#electEndpoints
     * (java.util.Map, org.ow2.petals.jbi.component.context.ComponentContext,
     * org.ow2.petals.jbi.messaging.exchange.MessageExchangeDecorator)
     */
    public void electEndpoints(
            final Map<ServiceEndpoint, TransportSendContext> electedDestinations,
            final ComponentContext sourceComponentContext,
            final MessageExchangeWrapper exchangeDecorator) throws RoutingException {
        this.log.call();

        // TODO handle transport selection by the message Exchange properties or
        // by the topology...

        for (TransportSendContext transportSendContext : electedDestinations.values()) {
            if (this.localContainerName.equals(transportSendContext.destination.getContainerName())) {
                transportSendContext.transport = Transporter.LOCAL_FRACTAL_TRANSPORTER;
            } else {
                transportSendContext.transport = Transporter.TCP_FRACTAL_TRANSPORTER;
            }

            if (exchangeDecorator.getProperty(RouterService.PROPERTY_ROUTER_SEND_ATTEMPT) != null) {
                transportSendContext.attempt = Short.valueOf((String) exchangeDecorator
                        .getProperty(RouterService.PROPERTY_ROUTER_SEND_ATTEMPT));
            } else {
                transportSendContext.attempt = this.attempt;
            }

            if (exchangeDecorator.getProperty(RouterService.PROPERTY_ROUTER_SEND_DELAY) != null) {
                transportSendContext.delay = Integer.valueOf((String) exchangeDecorator
                        .getProperty(RouterService.PROPERTY_ROUTER_SEND_DELAY));
            } else {
                transportSendContext.delay = this.delay;
            }
        }
    }

    /**
     * Start the Fractal component
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(java.util.logging.Logger.getLogger(Constants.FRACTAL_COMPONENT_LOGGER_NAME_TRANSPORT_RESOLVER));
        this.log.start();

        this.localContainerName = this.configurationService.getContainerConfiguration().getName();
        this.attempt = this.configurationService.getContainerConfiguration().getRouterSendAttempt();
        this.delay = this.configurationService.getContainerConfiguration().getRouterSendDelay();

        this.log.end();
    }

    /**
     * Stop the Fractal component
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws RoutingException {
        this.log.call();
    }

}
