/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.security;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;

import javax.xml.XMLConstants;
import javax.xml.namespace.QName;

import org.ow2.petals.jaas.GroupPrincipal;

/**
 * @author Christophe HAMERLING - EBM WebSourcing
 */
public class AuthorizationEntry {

    enum ROLE_TYPE {
        ADD, SET, REM
    }

    private Set<GroupPrincipal> accessControlList;

    private QName service;

    private String endpoint;

    private QName operation;

    private ROLE_TYPE type = ROLE_TYPE.ADD;

    /**
     * Creates a new instance of AuthorizationEntry
     */
    public AuthorizationEntry() {
    }

    /**
     * Creates a new instance of AuthorizationEntry
     * 
     * @param service
     * @param endpoint
     * @param operation
     * @param roles
     *            roles as CSV
     */
    public AuthorizationEntry(QName service, String endpoint, QName operation, String roles) {
        this(service, endpoint, operation, roles, ROLE_TYPE.ADD);
    }

    /**
     * Creates a new instance of AuthorizationEntry
     * 
     * @param service
     * @param endpoint
     * @param operation
     * @param roles
     * @param type
     */
    public AuthorizationEntry(QName service, String endpoint, QName operation, String roles,
            ROLE_TYPE type) {
        this.service = service;
        this.endpoint = endpoint;
        this.operation = operation;
        this.accessControlList = buildRoles(roles);
        this.type = type;
    }

    /**
     * @return the type
     */
    public ROLE_TYPE getType() {
        return type;
    }

    /**
     * @param type
     *            the type to set
     */
    public void setType(ROLE_TYPE type) {
        this.type = type;
    }

    /**
     * @return the endpoint
     */
    public String getEndpoint() {
        return endpoint;
    }

    /**
     * @param endpoint
     *            the endpoint to set
     */
    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    /**
     * @return the service
     */
    public QName getService() {
        return service;
    }

    /**
     * @param service
     *            the service to set
     */
    public void setService(QName service) {
        if (XMLConstants.NULL_NS_URI.equals(service.getNamespaceURI())
                && service.getPrefix() != null && service.getPrefix().length() > 0) {
            service = new QName(service.getPrefix(), service.getLocalPart());
        }
        this.service = service;
    }

    /**
     * @return the operation
     */
    public QName getOperation() {
        return operation;
    }

    /**
     * @param operation
     *            the operation to set
     */
    public void setOperation(QName operation) {
        this.operation = operation;
    }

    /**
     * @return the accessControlList
     */
    public Set<GroupPrincipal> getAcls() {
        return accessControlList;
    }

    /**
     * @param accessControlList
     *            the accessControlList to set
     */
    public void setAcls(Set<GroupPrincipal> acls) {
        this.accessControlList = acls;
    }

    /**
     * 
     * @param roles
     */
    public void setRoles(String roles) {
        this.accessControlList = buildRoles(roles);
    }

    /**
     * 
     * @return
     */
    public String getRoles() {
        StringBuffer sb = new StringBuffer();
        if (this.accessControlList != null) {
            for (Iterator<GroupPrincipal> iter = this.accessControlList.iterator(); iter.hasNext();) {
                GroupPrincipal p = iter.next();
                sb.append(p);
                if (iter.hasNext()) {
                    sb.append(",");
                }
            }
        }
        return sb.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "AuthorizationEntry[service=" + service + ", endpoint=" + endpoint + ", operation="
                + operation + ", roles=" + getRoles() + "]";
    }

    /**
     * 
     * @param roles
     * @return
     */
    private Set<GroupPrincipal> buildRoles(String roles) {
        Set<GroupPrincipal> s = new HashSet<GroupPrincipal>();
        StringTokenizer iter = new StringTokenizer(roles, ",");
        while (iter.hasMoreTokens()) {
            String name = iter.nextToken().trim();
            s.add(new GroupPrincipal(name));
        }
        return s;
    }
}
