/**
 * Copyright (c) 2005-2012 Fossil E-Commerce, 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader;

import java.io.IOException;
import java.net.URL;
import java.net.URLStreamHandlerFactory;
import java.util.List;

/**
 * This classLoader is used for a share library. 
 * <p>
 * The SharedLibraries-classloader
 * use it to find classes requested by a JBIcomponent-classloader. The
 * SharedLibraries-classloader has to call the protected method "findClass" on
 * it. So this classloader exposes the "findclass" method as a public method.
 * </p>
 * 
 * @since Petals 1.0
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class SharedLibraryClassLoader extends PetalsClassLoader {

    public SharedLibraryClassLoader(URL[] baseUrls,
            List<String> classpathLocations) throws IOException {
        super(baseUrls, classpathLocations);
    }

    public SharedLibraryClassLoader(URL[] baseUrls,
            List<String> classpathLocations, ClassLoader parent)
            throws IOException {
        super(baseUrls, classpathLocations, parent);
    }

    public SharedLibraryClassLoader(URL[] baseUrls,
            List<String> classpathLocations, ClassLoader parent,
            URLStreamHandlerFactory factory) throws IOException {
        super(baseUrls, classpathLocations, parent, factory);
    }

    /**
     * Expose the protected <code>findLoadedClass</code> method. Used by the
     * ShareLibraries class loader
     * 
     * @param className
     * @return
     */
    Class<?> findLoadedClazz(String className) {
        return super.findLoadedClass(className);
    }
}
