/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.transport.util;

import org.ow2.petals.launcher.api.service.Location;
import org.ow2.petals.microkernel.transport.Transporter;

/**
 * The Transport Context provides the information to transport the message
 * toward the destination with the proper protocol.
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 */
public class TransportSendContext {

    public TransportSendContext(final String subdomainName, final String containerName,
            final String componentName) {
        this(new Location(subdomainName, containerName, componentName));

    }

    public TransportSendContext(final Location destination) {
        this.destination = destination;
        this.attempt = 1;
        this.transport = Transporter.LOCAL_FRACTAL_TRANSPORTER;
    }

    public TransportSendContext(Location destination, long timeout, short attempt, int delay,
            String transportQOS) {
        this.destination = destination;
        this.timeout = timeout;
        this.attempt = attempt;
        this.delay = delay;
        this.transport = transportQOS;
    }

    /**
     * The destination
     */
    public Location destination;

    /**
     * The QOS of the transport
     */
    public String transport;

    /**
     * The send timeout
     */
    public long timeout;

    /**
     * Number of send attempt, if a failure occurs during the transfer
     */
    public short attempt;

    /**
     * Delay between send attempt
     */
    public int delay;
}
