/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.communication.jndi.tribe.connection;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;

import org.ow2.petals.microkernel.communication.jndi.agent.msg.request.RegistryRequest;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.request.RegistryRequest.RequestType;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.AckResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.ExceptionResponse;
import org.ow2.petals.microkernel.communication.jndi.agent.msg.response.RegistryResponse;

/**
 * Mock server socket.
 * @author ddesjardins - EBM WebSourcing
 */
public class ServerSocketThread extends Thread {

	protected ServerSocket serverSocket;

	public ServerSocketThread(int port, String host) {
		try {
			serverSocket = new ServerSocket();
			serverSocket.bind(new InetSocketAddress(host, port));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	protected boolean run;

	@Override
    public void run() {
		run = true;
		while (!serverSocket.isBound()) {
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
				// do nothing
			}
		}
		while (run) {
			try {
				Socket socket = serverSocket.accept();
				RegistryRequest request = null;
				RegistryResponse response = null;

				ObjectInputStream objectInputStream = new ObjectInputStream(
						socket.getInputStream());
				request = (RegistryRequest) objectInputStream.readObject();

				if (request.getType().equals(RequestType.ping)) {
					response = new AckResponse();
					response.setArg1("test");
				} else {
					response = new ExceptionResponse(new RuntimeException());
				}
				ObjectOutputStream objectOutputStream = new ObjectOutputStream(
						socket.getOutputStream());
				objectOutputStream.writeObject(response);
				objectOutputStream.flush();
				objectOutputStream.close();

			} catch (IOException e) {
                if (run) {
                    e.printStackTrace();
                }
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	public void terminate() {
		try {
			run = false;
			serverSocket.close();
		} catch (IOException e) {
			// do nothing
		} catch (Throwable e) {
			// do nothing
		}
	}

}
