/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.jbi.management.task.download.jetty.server;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * This class is used to store only the SOAP server related configuration values.
 * @author Christophe HAMERLING - EBM WebSourcing
 */
public class ServerConfig {

    /**
     * The port used for incoming SOAP requests
     */
    private int port = Constants.HttpServer.DEFAULT_HTTP_PORT;

    /**
     * The current interfaces which can be use for incoming listening
     */
    private List<InetAddress> addresses;

    /**
     * The host
     */
    private String host = Constants.HttpServer.DEFAULT_HTTP_HOST;

    /**
     * Restrict access to a particular network interface
     */
    private boolean restrict = false;

    /**
     * Default value is set to 255 (The jetty implementation default)
     */
    protected int jettyThreadMaxPoolSize = Constants.HttpServer.DEFAULT_HTTP_THREAD_POOL_SIZE_MAX;

    /**
     * Default value is set to 1 (The jetty implementation default)
     */
    protected int jettyThreadMinPoolSize = Constants.HttpServer.DEFAULT_HTTP_THREAD_POOL_SIZE_MIN;

    /**
     * Default value is set to 4
     */
    protected int jettyAcceptors = Constants.HttpServer.DEFAULT_HTTP_ACCEPTORS;

    /**
     * Creates a new instance of {@link ServerConfig}
     * 
     */
    public ServerConfig() {
        this.addresses = new ArrayList<InetAddress>();
    }

    /**
     * @return the port
     */
    public int getPort() {
        return this.port;
    }

    /**
     * @param port
     *            the port to set
     */
    public void setPort(final int port) {
        this.port = port;
    }

    /**
     * @param host
     *            the host to set
     */
    protected void addAddress(final InetAddress ia) {
        this.addresses.add(ia);
    }

    /**
     * Add all the {@link InetAddress}
     * 
     * @param addresses
     */
    protected void addAddresses(final Set<Inet4Address> addresses) {
        this.addresses.addAll(addresses);
    }

    /**
     * @return the jettyThreadMaxPoolSize
     */
    public int getJettyThreadMaxPoolSize() {
        return this.jettyThreadMaxPoolSize;
    }

    /**
     * @param jettyThreadMaxPoolSize
     *            the jettyThreadMaxPoolSize to set
     */
    public void setJettyThreadMaxPoolSize(final int jettyThreadMaxPoolSize) {
        this.jettyThreadMaxPoolSize = jettyThreadMaxPoolSize;
    }

    /**
     * @return the jettyThreadMinPoolSize
     */
    public int getJettyThreadMinPoolSize() {
        return this.jettyThreadMinPoolSize;
    }

    /**
     * @param jettyThreadMinPoolSize
     *            the jettyThreadMinPoolSize to set
     */
    public void setJettyThreadMinPoolSize(final int jettyThreadMinPoolSize) {
        this.jettyThreadMinPoolSize = jettyThreadMinPoolSize;
    }

    /**
     * @return the jettyAcceptors
     */
    public int getJettyAcceptors() {
        return this.jettyAcceptors;
    }

    /**
     * @param jettyAcceptors
     *            the jettyAcceptors to set
     */
    public void setJettyAcceptors(final int jettyAcceptors) {
        this.jettyAcceptors = jettyAcceptors;
    }

    /**
     * 
     * @return
     */
    public String getHost() {
        /*
         * if (this.host == null) { this.host = this.buildHost(); }
         */
        return this.host;
    }

    /**
     * Get the host. If restricted, get the first address of the addresses set.
     * If not restricted, try to get an address which is not the local one since
     * it can be used from external callers.
     * 
     * @return
     */
    /*
     * public String buildHost() { InetAddress result = null;
     * 
     * if (restrict) { // get the first host, no matter if it is the loopback
     * since it has // been configured by the user to be restricted ! The set
     * should // contains only the user address if (addresses != null &&
     * addresses.size() > 0) { result = addresses.get(0); } } else { // get the
     * first inet address which is not the loopback one since it // can be
     * called from external clients if (addresses != null && addresses.size() >
     * 0) { Iterator<InetAddress> iter = addresses.iterator(); while
     * (iter.hasNext() && result == null) { InetAddress addr = iter.next(); if
     * (!NetworkUtil.isLoopbackAddress(addr)) { result = addr; } } } } return
     * (result == null ? Constants.HttpServer.DEFAULT_HTTP_HOST :
     * result.getHostAddress()); }
     */

    /**
     * @return the restrict
     */
    public boolean isRestrict() {
        return restrict;
    }

    /**
     * @param restrict
     *            the restrict to set
     */
    public void setRestrict(boolean restrict) {
        this.restrict = restrict;
    }
}
