/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.system.classloader.locator;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.junit.Before;
import org.junit.Test;

/**
 * Test the DirLocator.
 * @author ddesjardins - EBM WebSourcing
 */
public class DirLocatorTest {

    private String baseDir;

    @Before
    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
    }

    @Test
    public void testHasFile() throws MalformedURLException, IOException {
        DirLocator dirLocator = new DirLocator(new URL("file:"
                + baseDir.replace(File.separator, "/") + "src/test/resources/"));
        assertTrue(dirLocator.hasFile("resourceCore.txt"));
    }

    @Test
    public void testListContent() throws MalformedURLException, IOException {
        DirLocator dirLocator = new DirLocator(new URL("file:"
                + baseDir.replace(File.separator, "/") + "src/test/resources/"));
        assertTrue((dirLocator.listContent(".").size() > 0));
    }

    @Test
    public void testExceptionNotExist() {
        try {
            new DirLocator(new URL("file:\\test%20bad_file"));
            fail();
        } catch (IOException e) {
            // do nothing
        }
    }

    @Test
    public void testExceptionNotDirectory() {
        try {
            new DirLocator(new URL("file:" + baseDir.replace(File.separator, "/")
                    + "src/test/resources/test.zip"));
            fail();
        } catch (IOException e) {
            // do nothing
        }
    }

}
